# evalpk - routines to control evaluation of arithmetic expression:
#	   estart, evali, iterms
#
# Copyright (c) 1988 by Michael J. Harrison
#
# Revised MJH, 17-Feb-86: Extend to long integers, fix syntax bug
# Revised MJH, 24-May-86: Support real CHARACTER variables
# Revised MJH, 10-Jan-87: Switch to str... string functions
# Revised MJH, 13-Apr-88: Return error if given null string
# Revised MJH,  3-Jan-91: Remove include of (now non-existent) dstax.i
# M.J.Harrison 10-Apr-98: Linux

include dmach
include dstd
include drmeta
include devali

#----------------------------------------------------------------------
# estart - process start of arithmetic expression
#----------------------------------------------------------------------

subroutine estart(tokstr, toksiz, ostack)
integer toksiz, ostack(STAKSZ)
character tokstr(toksiz)

intfunc rdtok

    integer ic

    repeat
	{
	ic = tokstr(1)
	ic = IAND(ic, BYTEMASK)
	if (ic == LPAREN)           # push ( we read
	    call epush(LPCODE, ostack)
	else if (ic == MINUS)       # unary minus
	    call epush(NEGCODE, ostack)
	else if (ic ^= PLUS)        # unary plus ignored
	    break

	} until (rdtok(tokstr, toksiz) == EOF) # break is normal loop end

    return
    end

#----------------------------------------------------------------------
# evali - evaluate arithmetic expression in integer variables
#----------------------------------------------------------------------

integer*4 function evali(tokstr, toksiz, errcd)
integer toksiz, errcd
character tokstr(toksiz)

intfunc*4 iterms

    integer ostack(STAKSZ)
    integer*4 vstack(STAKSZ)

    ostack(MAXSPSUB) = STAKSZ
    vstack(MAXSPSUB) = STAKSZ
    vstack(SPSUB) = INITSP
    ostack(SPSUB) = INITSP
    tokstr(1) = LPAREN			# initial ( to push
    call estart(tokstr, toksiz, ostack)
    if (tokstr(1) == NEWLINE) {		# null expression
	errcd = VXERRC			# flag "operand expected" error
	evali = 0			# and return zero
	}
    else {				# else evaluate expression
	errcd = 0
	evali = iterms(tokstr, toksiz, ostack, vstack, errcd)
	}

    return
    end

#----------------------------------------------------------------------
# iterms - process terms of arithmetic expression in integer variables
#----------------------------------------------------------------------

integer*4 function iterms(tokstr, toksiz, ostack, vstack, errcd)
integer toksiz, ostack(STAKSZ), errcd
integer*4 vstack(STAKSZ)
character tokstr(toksiz)

intfunc strchr, strlen
intfunc*4 epop4

    integer opcode
    logical sempty
    character t
    string oper "(+-*/ )!"  # (last char gets set to NL)
			    # must parallel streng (in doop)

    oper(strlen(oper)) = NEWLINE
    for (t = tokstr(1); t ^= NEWLINE & errcd <= 0; t = tokstr(1)) {
	call dovar(tokstr, toksiz, ostack, vstack, errcd)
	for (opcode = strchr(oper, tokstr(1)); opcode == RPCODE & errcd <= 0;
	    opcode = strchr(oper, tokstr(1)))	 # do right paren(s)
	    call doop(opcode, ostack, vstack, tokstr, toksiz, errcd)
	if (opcode < NLCODE & errcd <= 0) {	# leave end-of-expr till later
	    if (opcode > 1)			# actual operator
		call doop(opcode, ostack, vstack, tokstr, toksiz, errcd)
	    else
		errcd = OXERRC
	    }
	}
    if (errcd == 0) {	      # complete evaluation
	call doop(NLCODE, ostack, vstack, tokstr, toksiz, errcd)
	iterms = epop4(vstack, sempty)
	}
    else
	iterms = 0

    return
    end
