# evdopk - routines for doing operations for expression evaluation:
#	   doeval, doop, dovar, epop, epop4, epush, epush4
#
# Copyright (c) 1988 by Michael J. Harrison
#
# Revised MJH, 17-Feb-86: Extend to long integers, fix syntax bug
# Revised MJH, 24-May-86: Support real CHARACTER variables
# Revised MJH, 31-May-86: Use common type array
# Revised MJH, 12-Apr-88: Support new define
# Revised MJH,  9-Sep-88: For INTEGER*1 chars
# Revised MJH,  3-Jan-91: Remove include of (now non-existent) dstax.i
# M.J.Harrison 10-Apr-98: Linux

include dstd
include devali

#----------------------------------------------------------------------
# doeval - perform evaluation on stacks (integer arithmetic)
#----------------------------------------------------------------------

subroutine doeval(ostack, vstack, errcd)
integer ostack(ARB), errcd
integer*4 vstack(ARB)

intfunc epop
intfunc*4 epop4

    integer opcode
    logical sempty
    integer*4 var1, var2, result

    for (opcode = epop(ostack, sempty); opcode ^= LPCODE & ^sempty;
	opcode = epop(ostack, sempty)) {
	if (opcode == NEGCODE)	# unary op(s)
	    call epush4(-epop4(vstack, sempty), vstack)
	else {			# binary ops
	    var2 = epop4(vstack, sempty)
	    var1 = epop4(vstack, sempty)
	    if (opcode == PLSCODE)
		result = var1 + var2
	    else if (opcode == MINCODE)
		result = var1 - var2
	    else if (opcode == MULCODE)
		result = var1 * var2
	    else if (opcode == DIVCODE) {
		if (var2 == 0) {
		    errcd = ZDERRC
		    result = 0	# anything/0 = 0
		    }
		else
		    result = var1 / var2
		}
	    else
		call error("Bad operator code (doeval).")
	    call epush4(result, vstack)
	    }
	}

    if (opcode == LPCODE)	    # overshot, so
	call epush(LPCODE, ostack)  # put back "("
    else
	errcd = UPERRC

    return
    end

#----------------------------------------------------------------------
# doop - process operator of arithmetic expression in integers
#----------------------------------------------------------------------

subroutine doop(opcode, ostack, vstack, tokstr, toksiz, errcd)
integer opcode, ostack(ARB), toksiz, errcd
integer*4 vstack(ARB)
character tokstr(toksiz)

intfunc epop
intfunc rdtok

    integer junk, lastop, streng(8), topop
    logical sempty
    data streng/0,1,1,2,2,3,0,0/    # (,+,-,*,/, ,),NL
				    # must parallel oper (in iterms)

    if (opcode > MAXAOP) {	    # ')' or end-of-expr
	call doeval(ostack, vstack, errcd)
	lastop = epop(ostack, sempty) # pop '('
	if (opcode == NLCODE) {     # if finish, check
	    junk = epop(ostack, sempty)
	    if (^sempty)	    # if ops left, error
		errcd = UPERRC
	    }
	else			    # else advance
	    junk = rdtok(tokstr, toksiz)
	}
    else {
	topop = epop(ostack, sempty)
	call epush(topop, ostack)
	if (streng(opcode) <= streng(topop))
	    call doeval(ostack, vstack, errcd)
	call epush(opcode, ostack)
	junk = rdtok(tokstr, toksiz)
	}

    return
    end

#----------------------------------------------------------------------
# dovar - process operand of arithmetic expression in integers
#----------------------------------------------------------------------

subroutine dovar(tokstr, toksiz, ostack, vstack, errcd)
integer toksiz, ostack(ARB), errcd
integer*4 vstack(ARB)
character tokstr(toksiz)

intfunc*4 ctoi
logfunc alldig
intfunc rdtok

include ctype

    integer i, junk
    integer*4 i4

    call estart(tokstr, toksiz, ostack)     # deal with (, unary ops
    i = tokstr(1)
    if (type(1 + IAND(i, CHARMASK)) == ALPHA)
	errcd = UVERRC
    else if (^alldig(tokstr))
	errcd = VXERRC
    else {
	i = 1		    # i needed because altered by ctoi
	i4 = ctoi(tokstr, i)
	call epush4(i4, vstack)
	junk = rdtok(tokstr, toksiz)
	}

    return
    end

#----------------------------------------------------------------------
# epop - pop 1-word item from self-defining stack
#----------------------------------------------------------------------

integer function epop(stack, sempty)
integer stack(ARB)
logical sempty

    integer sp

    sp = stack(SPSUB)
    if (sp <= INITSP) {     # if stack empty
	epop = 0	    # return zero
	sempty = YES	    # and flag empty
	}
    else {		    # else
	epop = stack(sp)    # return value
	sempty = NO	    # flag not empty
	sp = sp - 1	    # upstack
	}
    stack(SPSUB) = sp

    return
    end

#----------------------------------------------------------------------
# epop4 - pop long int item from self-defining stack
#----------------------------------------------------------------------

integer*4 function epop4(stack, sempty)
integer*4 stack(ARB)
logical sempty

    integer sp

    sp = stack(SPSUB)
    if (sp <= INITSP) {     # if stack empty
	epop4 = 0	    # return zero
	sempty = YES	    # and flag empty
	}
    else {		    # else
	epop4 = stack(sp)   # return value
	sempty = NO	    # flag not empty
	sp = sp - 1	    # upstack
	}
    stack(SPSUB) = sp

    return
    end

#----------------------------------------------------------------------
# epush - push 1-word item onto self-defining stack
#----------------------------------------------------------------------

subroutine epush(item, stack)
integer item, stack(ARB)

    integer sp

    sp = stack(SPSUB)
    sp = sp + 1
    if (sp > stack(MAXSPSUB))
	call error ("Stack overflow (epush).")
    else
	stack(sp) = item
    stack(SPSUB) = sp

    return
    end

#----------------------------------------------------------------------
# epush4 - push long int item onto self-defining stack
#----------------------------------------------------------------------

subroutine epush4(item, stack)
integer*4 item, stack(ARB)

    integer sp

    sp = stack(SPSUB)
    sp = sp + 1
    if (sp > stack(MAXSPSUB))
	call error ("Stack overflow (epush4).")
    else
	stack(sp) = item
    stack(SPSUB) = sp

    return
    end
