#  forpk - routines for "for" statement:
#	   forcod, forfld, forini, forrei, forsta, fortst, initfs
#
# Copyright (c) 1987 by Michael J. Harrison
#
#  Revised MJH, 24-May-86: Support real CHARACTER variables
#  Revised MJH, 10-Jan-87: Switch to common tokbuf array
#  Revised MJH, 14-Mar-87: Bug fix: forfld failed with "for (...;\n..."

include dstd
include drmax
include drmeta

#---------------------------------------------------------------------
# forcod - generate code for start of "for" statement
#---------------------------------------------------------------------

subroutine forcod(lab)
integer lab

include ctoken
intfunc labgen

    call forini(tokbuf, MAXTOK) 	# 'init' part
    lab = labgen(3)			# generate 3 labels
    call fortst(tokbuf, MAXTOK, lab)	# 'test' part
    call forrei(tokbuf, MAXTOK) 	# stack 'reinit' part

    return
    end

#---------------------------------------------------------------------
# forfld - deal with first or second field of "for" statement
#---------------------------------------------------------------------

subroutine forfld(tpeek, tokstr, toklen)
integer tpeek			# Peek ahead token
integer toklen
character tokstr(toklen)	# Its string is here

intfunc gettok

    integer nlpar, t

    nlpar = 0
    t = tpeek
    while (t ^= EOF & t ^= SEMICOL) {
	if (t == LBRACE | t == RBRACE) {
	    call synerr("Illegal brace in 'for' stmt.")
	    break
	    }
	if (t ^= NEWLINE) {		# ignore NEWLINE
	    if (t == LPAREN)
		nlpar = nlpar + 1
	    else if (t == RPAREN)
		nlpar = nlpar - 1
	    #else nothing special
	    call outstr(tokstr)
	    }
	t = gettok(tokstr, toklen)
	}
    if (nlpar ^= 0)
	call synerr("Unbalanced parentheses in 'for' stmt.")

    return
    end

#---------------------------------------------------------------------
# forini - do 'init' field of "for" statement
#---------------------------------------------------------------------

subroutine forini(tokstr, toklen)
integer toklen
character tokstr(toklen)

intfunc gettok

    integer t

    if (gettok(tokstr, toklen) ^= LPAREN)
	call synerr("Missing '(' in 'for' stmt.")
    else {
	t = gettok(tokstr, toklen)
	if (t == SEMICOL)
	    call outcon(0)		# CONTINUE iff label and null 'init'
	else {
	    call outtab
	    call forfld(t, tokstr, toklen)
	    call outdon 		# finish off
	    }
	}

    return
    end

#---------------------------------------------------------------------
# forrei - stack reinitialize statement for "for"
#---------------------------------------------------------------------

subroutine forrei(tokstr, toklen)
integer toklen
character tokstr(toklen)

intfunc gettok
include cforda

    integer nlpar, i, j, t

    forsp = forsp + 1
    if (forsp > MAXFOR)
	call error("Stack overflow (forrei).")
    forptr(forsp) = fdfree
    j = fdfree
    nlpar = 0
    while (nlpar >= 0) {
	t = gettok(tokstr, toklen)
	if (t == LBRACE | t == RBRACE |
	    t == SEMICOL | t == EOF) {
	    call synerr("Illegal character in 'for' stmt.")
	    break
	    }
	if (t ^= NEWLINE) {		# ignore NEWLINE's
	    if (t == LPAREN)
		nlpar = nlpar + 1
	    else if (t == RPAREN)
		nlpar = nlpar - 1
	    #else nothing special
	    i = 1
	    while (tokstr(i) ^= EOS) {	# copy token (except EOS)
		reinit(j) = tokstr(i)
		i = i + 1
		j = j + 1
		if (j > MAXREINIT)
		    call error("Storage overflow (forrei).")
		}
	    }
	}
    reinit(j - 1) = EOS 		# overwrite final ")"
    fdfree = j

    return
    end

#---------------------------------------------------------------------
# forsta - generate code for end of "for" statement
#---------------------------------------------------------------------

subroutine forsta(lab)
integer lab

intfunc strlen
include cforda

    if (forsp <= 0)
	call error("Stack underflow (forsta).")
    fdfree = forptr(forsp)
    forsp = forsp - 1
    call outnum(lab)			# reinit label
    call outtab
    if (strlen(reinit(fdfree)) > 0) {
	call outstr(reinit(fdfree))	# reinit stmt
	call outdon
	}
    call outgo(lab + 2) 		# loop-back GOTO
    call outcon(lab +1 )		# loop exit label

    return
    end

#---------------------------------------------------------------------
# fortst - generate code for test for "for" statement
#---------------------------------------------------------------------

subroutine fortst(tokstr, toklen, lab)
integer toklen, lab
character tokstr(toklen)

intfunc gettok

    integer t
    string ifnot "IF(.NOT.(", closep "))"

    t = gettok(tokstr, toklen)		# peek ahead
    if (t == SEMICOL)
	call outcon(lab + 2)		# null test field - use CONTINUE
    else {
	call outnum(lab + 2)		# label
	call outtab
	call outstr(ifnot)		# "IF(.NOT.("
	call forfld(t, tokstr, toklen)
	call outstr(closep)		# "))"
	call outgo(lab + 1)		# "GOTOlab+1" (loop exit)
	}

    return
    end

#----------------------------------------------------------------------
# initfs - initialize 'for' and 'string' data areas
#---------------------------------------------------------------------

subroutine initfs

include cforda
include cstrda

    fdfree = 1
    forsp = 0
    sdfree = 1
    return
    end
