#-----------------------------------------------------------------------
#
#  ifdpk - Process ifdef, ifndef, and endif statements
#	   Contains: endif, ifeval
#
#  Written MJH, 14-Jun-86:
#  Revised MJH, 29-Dec-90: Adapt to changed interface for lookup
#  M.J.Harrison 23-Apr-95: In ifeval, replace call to flushs (which had
#			   an extra argument anyway) with call to flushl
#
#-------------- Copyright (C) 1986 by Michael J. Harrison --------------

include dstd
include dlex
include drmax		# for MAXTOK

#-----------------------------------------------------------------------
#  endif - Process endif statement
#-----------------------------------------------------------------------

subroutine endif(tokstr, toksiz, ifdlvl)
integer toksiz
character tokstr(toksiz)
integer ifdlvl

intfunc flushl

    integer junk

    if (ifdlvl > 0)
	ifdlvl = ifdlvl - 1
    else
	call synerr("Superfluous 'endif' ignored.")
    junk = flushl(tokstr,toksiz)

    return
    end # of endif

#-----------------------------------------------------------------------
#  ifeval - Process ifdef and ifndef statements
#-----------------------------------------------------------------------

subroutine ifeval(token, tokstr, toksiz, ifdlvl)
integer token		# Indicates either ifdef or ifndef
integer toksiz
character tokstr(toksiz)
integer ifdlvl		# Nesting level, may be updated

intfunc flushl, skipl
intfunc lex, lookup
intfunc rdtok		# We need raw tokens

    character cdummy(1)
    integer nowlvl, t
    logical wantit	# TRUE if ifdef, FALSE if ifndef
    logical gotit	# TRUE if token defined, else FALSE

    wantit = (token == LEXIFDEF)
    t = rdtok(tokstr, toksiz)
    if (t == EOF)
	call synerr("Unexpected end-of-file.")
    else if (t == NEWLINE)
	gotit = NO	# No token is equivalent to not defined
    else
	gotit = (lookup(tokstr, cdummy, NO) != 0)

    if (t ^= EOF)
	{
	nowlvl = ifdlvl
	ifdlvl = ifdlvl + 1
	if (t ^= NEWLINE)
	    t = flushl(tokstr, toksiz)

	# If ifdef and not defined or ifndef and defined, we chew
	# up lines until a matching endif is encountered.
	if (t ^= EOF & ((wantit & ^gotit) | (^wantit & gotit)))
	    {
	    while (ifdlvl > nowlvl)
		{
		t = lex(tokstr, toksiz)	# Peek for ifdef, ifndef, endif
		if (t == LEXIFDEF | t == LEXIFNDEF)
		    ifdlvl = ifdlvl + 1
		else if (t == LEXENDIF)
		    ifdlvl = ifdlvl - 1
#		call pbstr(tokstr)
		if (skipl(tokstr, toksiz) == EOF)
		    {
		    call synerr("Missing 'endif'.")
		    break
		    }
		}
	    }
	}

    return
    end # of ifeval
