#  incopn - Open file for 'include' statement using INCLUDE= paths
#
# Copyright (c) 1987 by Michael J. Harrison
#
#  Notes:
#	If there is an INCLUDE environment variable, ONLY those paths
#	it specifies are searched (in order, left to right), and NOT
#	the current directory (unless it's actually specified).
#	Otherwise, ONLY the current directory is searched.
#	If we are given an explicit drive or an absolute pathname,
#	only that is used, whatever the INCLUDE paths are.
#
#  Written MJH, 15-Dec-86: Copyright (C) 1986 by Michael J. Harrison
#  Revised MJH, 28-Feb-87: Allow each path optionally to end in "\"
#			   Handle explicit drive better
# Revised MJH, 16-Apr-88: Replace parameter statement by define
# M.J.Harrison  9-Apr-98: For Linux
# Mike Harrison 04/19/2014: For MinGW

include dmach	    # for LENxxx
include dstd
include drmeta
include dstdio	    # for INPUT

define MAXPATH (LENPATH + LENFULLFN)

integer function incopn(incnam)
character incnam(ARB)		# Name (+ ext.) of include file
#
# Returns: File handle (>= 0) if success, else < 0

intfunc open, rpgete, strcpy

    integer fid, i, j, junk, len
    character curpth(MAXPATH)	# Holds current pathname
    character pathst(MAXENVSTR) # Holds string of INCLUDE=string
    character pdelim		# Holds ; for DOS, : for UNIX
    string incl "INCLUDE"

    # Set up path delimiter.
    pdelim = COLON
ifdef MSFORT4
    pdelim = SEMICOL
endif

    # If explicit drive or absolute file path is given,
    # or no INCLUDE=string is defined in the environment,
    # just try open with name passed to us.
    #
ifdef MSFORT4
    if (incnam(1) == PATHSEP | incnam(2) == COLON)
endif
ifndef MSFORT4
    if (incnam(1) == PATHSEP)
endif
	len = 0
    else
	len = rpgete(incl, pathst)

    if (len == 0)
	fid = open(incnam, INPUT)
    else {

	# For each specified path, append \ or / (if not already there)
	# and incnam, and try to open.
	#
ifdef MSFORT4
	call trn2uc(pathst)
endif
	# Canoncalize the paths to UNIX form. This is to deal with MinGW which, when
	# running under bash (e.g. from make) returns the paths in Windows form.
	for (i = 1; pathst(i) ^= EOS; i = i + 1) {
	    if (pathst(i) == SEMICOL) {
		pathst(i) = COLON
	    } else if (pathst(i) == BACKSL) {
		pathst(i) = SLASH
	    }
	}

	i = 1
	fid = -1			# To get into loop
	while (i <= len & fid < 0) {
	    j = 1
	    while (i <= len & pathst(i) ^= pdelim) {
		curpth(j) = pathst(i)
		i = i + 1
		j = j + 1
	    }
	    i = i + 1			# Step over ; delimiter

	    if (j == 1 | curpth(j-1) ^= PATHSEP) {
		curpth(j) = PATHSEP
		j = j + 1
	    }
	    junk = strcpy(curpth(j), incnam)
	    fid = open(curpth, INPUT)
	}
    }

    incopn = fid
    return
    end 	# of incopn
