# kwpk - routines dealing with Ratfor keywords: ckeybd, initkw, seekw
#
# Copyright (c) 1988 by Michael J. Harrison
#
# Revised MJH, 13-Jun-86: Add ifdef, ifndef, endif
# Revised MJH, 14-Jun-86: Improve efficiency of search
# Revised MJH,	9-Jan-87: More compact method for initkw, using one
#			    keyword string and one array of codes.
# Revised MJH, 16-Apr-88: Retire parameter statement
# Revised MJH,	9-Sep-88: For INTEGER*1 chars
# Revised MJH,	3-Jan-91: Use CHARMASK
# M.J.Harrison 10-Apr-98: Linux

include dmach
include dstd
include dlex
include drmax
include dmetac

define MAXKWL 8 	# length of longest keyword (INCLUDE), plus EOS
define MAXKWS 16	# number of Ratfor keywords
define MAXKWT 92	# total number of chars in keyword table (count 'em)

#----------------------------------------------------------------------
# ckeybd - block data subprogram to initialize keyword list and codes
#----------------------------------------------------------------------

block data ckeybd

include ckeywd

    # Obviously, the orders of keywds and kcodes must correspond!
    # For maximum search efficiency, all keywords starting with the
    # same letter should be adjacent to each other.
    #
    # If you change these arrays, be sure to keep
    # MAXKWL, MAXKWS, and MAXKWT up to date.
    #
    # Note that the ice is thin here. keywds is declared as INTEGER*1,
    # so strictly speaking we should define e.g. UCB as 66 and write
    # "data keywds /UCB ..."; but characters seem to work OK and are
    # more legible. g77 gives a warning with -fpedantic.
    #
    # Mike Harrison 10/15/2006: The ice broke with f95. I could not be
    #				bothered to define UCA etc., so I just
    #				used the raw decimal numbers instead.

#   data keywds /'B', 'R', 'E', 'A', 'K', EOS,
#		 'D', 'E', 'F', 'I', 'N', 'E', EOS,
#		 'D', 'O', EOS,
#		 'E', 'L', 'S', 'E', EOS,
#		 'E', 'N', 'D', 'I', 'F', EOS,
#		 'F', 'O', 'R', EOS,
#		 'I', 'F', EOS,
#		 'I', 'F', 'D', 'E', 'F', EOS,
#		 'I', 'F', 'N', 'D', 'E', 'F', EOS,
#		 'I', 'N', 'C', 'L', 'U', 'D', 'E', EOS,
#		 'N', 'E', 'X', 'T', EOS,
#		 'R', 'E', 'P', 'E', 'A', 'T', EOS,
#		 'S', 'T', 'R', 'I', 'N', 'G', EOS,
#		 'U', 'N', 'D', 'E', 'F', EOS,
#		 'U', 'N', 'T', 'I', 'L', EOS,
#		 'W', 'H', 'I', 'L', 'E', EOS/
    data keywds /66, 82, 69, 65, 75, NUL,
		 68, 69, 70, 73, 78, 69, NUL,
		 68, 79, NUL,
		 69, 76, 83, 69, NUL,
		 69, 78, 68, 73, 70, NUL,
		 70, 79, 82, NUL,
		 73, 70, NUL,
		 73, 70, 68, 69, 70, NUL,
		 73, 70, 78, 68, 69, 70, NUL,
		 73, 78, 67, 76, 85, 68, 69, NUL,
		 78, 69, 88, 84, NUL,
		 82, 69, 80, 69, 65, 84, NUL,
		 83, 84, 82, 73, 78, 71, NUL,
		 85, 78, 68, 69, 70, NUL,
		 85, 78, 84, 73, 76, NUL,
		 87, 72, 73, 76, 69, NUL/

    data kcodes /LEXBREAK,
		 LEXDEFINE,
		 LEXDO,
		 LEXELSE,
		 LEXENDIF,
		 LEXFOR,
		 LEXIF,
		 LEXIFDEF,
		 LEXIFNDEF,
		 LEXINCLUDE,
		 LEXNEXT,
		 LEXREPEAT,
		 LEXSTRING,
		 LEXUNDEF,
		 LEXUNTIL,
		 LEXWHILE/

    data kwpbeg /0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0/

    end 	# of ckeybd

#----------------------------------------------------------------------
# initkw - initialize keyword list for Ratfor
#----------------------------------------------------------------------

subroutine initkw

include ckeywd

    integer ic, ikwp, iw, letbas

    letbas = ICHAR('A') - 1	# Convert initial letter to index to kwpbeg
    iw = 1
    for (ic = 1; ic <= MAXKWS; ic = ic + 1) {
	kwptr(ic) = iw
	ikwp = keywds(iw) - letbas
	if (kwpbeg(ikwp) == 0)
	    kwpbeg(ikwp) = ic
	kwpend(ikwp) = ic
	while (keywds(iw) ^= EOS)
	    iw = iw + 1
	iw = iw + 1
	}

    return
    end 	# of initkw

#----------------------------------------------------------------------
# seekw - look up keyword in table, return corresponding code
#----------------------------------------------------------------------

logical function seekw(token, code)
character token(ARB)
integer code

include ckeywd
charfunc touppr
intfunc strcmp

    integer i, j, k, s
    character c, uctok(MAXKWL)

    seekw = NO

    # Look at (upper-cased) initial character to see if match possible
    c = touppr(token(1))
    if (c >= ICHAR('A') & c <= ICHAR('Z')) {  # Must be letter
	j = c
	j = IAND(j, BYTEMASK) - ICHAR('A') + 1
	i = kwpbeg(j)
	if (i > 0) {		# Must have beginning entry in kwptr
	    # Match possible.  Make local U/C copy, detecting no match if
	    # longer than longest keyword, then look from kwpbeg to kwpend.
	    uctok(1) = c
	    k = 2
	    s = 2
	    repeat {
		c = token(k)
		uctok(s) = touppr(c)
		k = k + 1
		s = s + 1
		} until (c == EOS | s > MAXKWL)
	    if (c == EOS) {
		k = kwpend(j)
		while (i <= k & strcmp(uctok, keywds(kwptr(i))) ^= 0)
		    i = i + 1
		if (i <= k) {	    # Found
		    code = kcodes(i)
		    seekw = YES
		    }
		}
	    }
	}

    return
    end 	# of seekw
