# ngetcpk - routines for getting/ungetting characters:
#	    clnget, initio, ngetc, opnget, pbstr, putbak
#
# Copyright (c) 1988 by Michael J. Harrison
#
# Revised MJH, 12-Apr-88: Add putbak mark (for new define)
# Revised MJH,  9-Sep-88: For INTEGER*1 chars
# Revised MJH, 11-Dec-90: Implement ANSI C macro replacement semantics
# M.J.Harrison 10-Apr-98: Linux

include dmach
include dstd
include ddefio
include drmax

#----------------------------------------------------------------------
# clnget - close the current file for ngetc/putbak
#----------------------------------------------------------------------
# Revised MJH, 22-Dec-85: Change to function, and return popped fid
# M.J.Harrison 05/30/1999: Avoid compiler warning (g77, -O2)

integer function clnget(fid)	# Return "previous" fid
integer fid

include cdefio

    clnget = 0
    if (fid ^= fidstk(stackp))
	call error("Fid mismatch (clnget).")
    else if (bpstk(stackp) > bpstk(stackp-1))
	call error("Putback chars not all read (clnget).")
    else {
	stackp = stackp - 1
	if (stackp <= 0)
	    call error("Fid stack underflow (clnget).")
	else
	    clnget = fidstk(stackp)
	}

    return
    end

#----------------------------------------------------------------------
# initio - initialize the input/output mechanism
#----------------------------------------------------------------------

subroutine initio   # no arguments -- works off COMMON

include cdefio
include cline
include coutln

    maxbp = 0	    # in cdefio
    prevgc = EOF
    stackp = 1
    fidstk(1) = -1
    bpstk(1) = 0
    linect = 1	    # in cline
    exptok = YES
    outp = 0	    # in coutln

    return
    end

#----------------------------------------------------------------------
# ngetc - get next (possibly pushed back) character
#----------------------------------------------------------------------

integer function ngetc(c)	# Return integer value
character c			# Also put character here (if not EOF)

intfunc getch
include cdefio
include cline

    integer ic

    # If EOF already encountered, return EOF.
    # If no stacked characters, get next character; if EOF, return EOF
    # and remember; otherwise, return the character, count lines, list.
    # If stacked characters, pop and return top character.
    # Note that EOF is never stacked.
    # Increment line count for each character preceded by NEWLINE.

    if (bpmark(stackp) == EOF) {
	ngetc = EOF
	c = EOS
	}
    else if (bpstk(stackp) == bpmark(stackp)) {
	bpmark(stackp) = EOPUTBK
	ngetc = EOPUTBK
	c = EOS
	}
    else if (bpstk(stackp) <= bpstk(stackp - 1)) {  # no chars on stack
	if (fidstk(stackp) == PSEUDOFID)	# pseudo-EOF for macro stack
	    ic = EOF
	else
	    ic = getch(c, fidstk(stackp))
	if (ic == EOF) {
	    ngetc = EOF
	    bpmark(stackp) = EOF
	    }
	else {
	    ngetc = ic
	    if (prevgc == NEWLINE & stackp == 2)    # stackp == 2 iff main input
		linect = linect + 1
	    prevgc = ic
	    if (lsfid > 0)
		call putch(ic, lsfid)	    # write source listing
	    }
	}
    else {			    # stacked chars
	c = buf(bpstk(stackp))
	ngetc = c
	ngetc = IAND(ngetc, BYTEMASK)
	bpstk(stackp) = bpstk(stackp) - 1
	}

    return
    end

#----------------------------------------------------------------------
# opnget - open a new file for ngetc/putbak
#----------------------------------------------------------------------

subroutine opnget(fid)
integer fid

include cdefio

    stackp = stackp + 1
    if (stackp > MAXFILES)
	call error("Too many files open at once (opnget).")
    else {
	fidstk(stackp) = fid
	bpmark(stackp) = EOPUTBK
	bpstk(stackp) = bpstk(stackp - 1)   # buffer base for this level
	}

    return
    end

#----------------------------------------------------------------------
# pbmark - mark current putbak position
#----------------------------------------------------------------------

subroutine pbmark()

include "cdefio.i"

    if (bpmark(stackp) ^= EOPUTBK)
	call error("Invalid call to pbmark.")
    else
	bpmark(stackp) = bpstk(stackp)

    return
    end

#----------------------------------------------------------------------
# pbstr - push string back onto input
#----------------------------------------------------------------------
# Revised MJH, 10-Jan-87: Switch to str... string functions

subroutine pbstr(in)
character in(ARB)

intfunc strlen

    integer i

    i = strlen(in)
    while (i > 0) {
	call putbak(in(i))
	i = i - 1
	}

    return
    end

#----------------------------------------------------------------------
# putbak - push character back onto input
#----------------------------------------------------------------------

subroutine putbak(c)
character c

include cdefio

    bpstk(stackp) = bpstk(stackp) + 1
    if (bpstk(stackp) > maxbp)	    # Keep up max depth used
	maxbp = bpstk(stackp)
    if (bpstk(stackp) > BUFSIZE)
	call error("Buffer overflow (putbak).")
    else
	buf(bpstk(stackp)) = c

    return
    end
