# outpk - output routines for RF Ratfor preprocessor:
#	  outch, outcon, outgo, outnum, outstr, outtab
#
# Copyright (c) 1988 by Michael J. Harrison
#
# NOTE: outdon is in a file of its own, as it is somewhat OS dependent
#
# Revised MJH, 16-Apr-88: New define syntax
# Revised MJH,  8-Sep-88: Changes to outch calls for INTEGER*1 chars
# M.J.Harrison 10-Apr-98: Linux

include dstd
include drmax
include drmeta

#----------------------------------------------------------------------
# outch - put one character into output buffer
#----------------------------------------------------------------------

subroutine outch(c)
character c

include coutln

    integer i

    if (outp >= 72) {	# continuation card
	call outdon
	i = 1
	while (i < 6) {
	    outbuf(i) = BLANK
	    i = i + 1
	    }
	outbuf(6) = STAR
	outp = 6
	}
    outp = outp + 1
    outbuf(outp) = c

    return
    end

#----------------------------------------------------------------------
# outcon - output "n CONTINUE"
#----------------------------------------------------------------------

subroutine outcon(n)
include coutln

    integer n
    string contin "CONTINUE"

    if (n > 0) {
	call outnum(n)
	call outtab
	}
    if (outp >= 6) {	    # don't output unlabelled CONTINUE
	call outstr(contin)
	call outdon
	}

    return
    end

#----------------------------------------------------------------------
# outgo - output "GOTO n"
#----------------------------------------------------------------------

subroutine outgo(n)
integer n

    string goto "GOTO"

    call outtab
    call outstr(goto)
    call outnum(n)
    call outdon

    return
    end

#----------------------------------------------------------------------
# outnum - output decimal number
#----------------------------------------------------------------------

subroutine outnum(n)
integer n

integer itoc

    integer*4 i4
    integer i, len
    character chars(MAXCHARS)

    i4 = n
    len = itoc(i4, chars, MAXCHARS)
    i = 1
    while (i <= len) {
	call outch(chars(i))
	i = i + 1
	}

    return
    end

#----------------------------------------------------------------------
# outstr - output string, translate relational and logical operators
#----------------------------------------------------------------------
# Revised MJH, 13-Jun-81: translate lower case letter not in quoted string
#			  to upper case, if so requested (by tranuc)
# Revised MJH, 10-Jan-87: Switch to str... string functions

subroutine outstr(str)
character str(1)

include coutln	    # for tranuc
charfunc cnvesc, touppr
intfunc strchr

    integer i, j
    character c, cq, period, squote

    define PAIRMAX 4
    define NOPS 9
    define FRELOPLEN 4
    string rrelop "><^=&|",
	   frelop "GE. LE. NE. EQ. AND.OR. GT. LT. NOT."

    squote = SQUOTE
    period = PERIOD
    for (i = 1; str(i) ^= EOS; i = i + 1) {
	c = str(i)
	if (c == SQUOTE | c == DQUOTE) {
	    call outch(squote)          # initial quote of string
	    cq = c
	    for (i =i + 1; str(i) ^= cq; i = i + 1) {  # output string
		c = str(i)
		if (c == ESCFLAG)       # convert \... to char
		    c = cnvesc(str, i)
		call outch(c)
		if (c == SQUOTE)
		    call outch(squote)  # make pair of quotes
		}
	    call outch(squote)  	# final quote of string
	    }
	else {
	    j = strchr(rrelop, c)
	    if (j == 0) {		    # not composite or relational
		if (c == ESCFLAG)           # but look for \...
		    c = cnvesc(str, i)
		if (tranuc)		    # if required, trans to UC
		    c = touppr(c)
		call outch(c)
		}
	    else {			    # may be composite or relational
		if (j > 0 & j <= PAIRMAX) {
		    if (str(i + 1) == EQUALS)   # it's composite
			i = i + 1		# skip over
		    else			# it's just relational (or '=')
			j = j + PAIRMAX + 2	# so point at right place
		    }
		if (j > NOPS)			# Just '=' all alone
		    call outch(c)		#  doesn't get translated
		else {				# Translate
		    call outch(period)
		    for (j = 1 + FRELOPLEN * (j - 1); frelop(j) ^= PERIOD;
			j = j + 1)
			call outch(frelop(j))
		    call outch(period)
		    }
		} # if possible composite
	    } # if quote
	} # for i

    return
    end

#----------------------------------------------------------------------
# outtab - get past column 6
#----------------------------------------------------------------------

subroutine outtab

include coutln

    character blank

    blank = BLANK
    while (outp < 6)
       call outch(blank)

    return
    end
