# rf - Main program for Ratfor for MS-DOS, Linux, Mac OS X. MinGW
#
# Copyright (c) 1979-2017 by Michael J. Harrison

include "dstd.i"
include "dstdio.i"
include "dmach.i"
include "drmax.i"
include "drmeta.i"

#=======================================================================
# main - Call ratfor main program, return error code to DOS
#=======================================================================

# This extra layer of subroutine dates from some early version of MS
# FORTRAN (3.30?) and provided a way to avoid linking a lot of
# unnecessary code from the FORTRAN run-time library.

#  Revised MJH, 14-Mar-87: Changes for Microsoft FORTRAN 4.00

intfunc ratfor

    integer dummy

    dummy = 0
    call mfretc(ratfor(dummy))	# Return code to DOS for errorlevel test

    end # of main

#-----------------------------------------------------------------------
#
#  ratfor - Ratfor main program for "standard" Ratfor (Bell Labs style)
#
#  Purpose:  Process Ratfor source to FORTRAN source
#
#  Method:   See Brian W. Kernighan and P.J. Plauger, "Software Tools"
#		Addison-Wesley, 1976 (ISBN 0-201-03669-X)
#
#  Written MJH, 14-Sep-85: Ported from VM/CMS version of 1984
#  Revised MJH,  2-Feb-86: Use standard I/O if no argument given
#  Revised MJH, 17-Feb-86: Long integer evaluation, bugs fixed
#  Revised MJH,  6-Jun-86: STDERR changed to STDOUT
#  Revised MJH, 16-Dec-86: 3.20 New version for INCLUDE=
#				Also, option now precedes filename
#  Revised MJH, 10-Jan-87: 3.21 New version with longer max token length
#  Revised MJH, 16-Feb-87: 3.22 Now consistently uses rp...., not pf....
#  Revised MJH, 27-Feb-87: 3.23 Bug Fix - flush rest of 'include' line
#				before switching to new source stream (!)
#  Revised MJH, 28-Feb-87: 3.24 Bug Fix - allow INCLUDE= path optionally
#				to end with "\"
#  Revised MJH, 14-Mar-87: 4.00 Version for Microsoft FORTRAN 4.00
#  Revised MJH, 20-Jul-87: 4.01 Version for Microsoft FORTRAN 4.01
#  Revised MJH,  4-Aug-87: 4.10 Output FORTRAN-77 block IF for Ratfor if
#  Revised MJH, 19-Aug-87: 4.11 Bug Fix for uninst
#  Revised MJH, 25-Dec-87: 4.12 Unified file system, new build
#  Revised MJH, 10-Apr-88: 4.20 C syntax for define, and eval result
#				Retire parameter statement
#  Revised MJH,  8-Sep-88: 4.30 INTEGER*1 used for character type
#  Revised MJH,  1-Jan-91: 4.40 New macro replacement like ANSI C
#  M.J.Harrison 24-Apr-95: 4.41 Fix bug with ifeval/flushs
#  M.J.Harrison  5-Apr-98: 4.42 Fix bug with null macro replacement
#  M.J.Harrison  8-Apr-98: 4.43 Changes for Linux
#  M.J.Harrison 05/29/1999: 4.44.1 Changes for g77
#  Mike Harrison 01/20/2005: 4.44.2 Fix calls to strchr(s, FILSEP) so that
#			     character (INTEGER*1) is passed, not INTEGER
#  Mike Harrison 10/15/2006: 4.44.3 Port to GNU f95
#  Mike Harrison 04/06/2014: 4.44.4 Version for gfortran (Linux, OS X, MinGW)
#  Mike Harrison 12/30/2016: 4.44.5 Use INTEGER*8 offset for seek for x86-64
#  Mike Harrison 12/28/2017: 4.44.5 Clean up Makefiles, use __MINGW32__ predef for MinGW
#
#-------------- Copyright (C) 1985 - 2017 by Michael J. Harrison -------

define MAXPATH (LENPATH + LENFULLFN)

integer function ratfor(dummy)
integer dummy		# FORTRAN demands this for function

intfunc getarg, open, strchr, strcpy		# Primitives, utilities
intfunc clnget, labgen				# Ratfor system
subrtn	cant, close				# Primitives, utilities
ifdef MSFORT4
subrtn trn2uc
endif
subrtn	initfs, initgp, initio, initkw, initlu	# Ratfor system
subrtn	opnget, outdon, outstr, rfdate, rfmkfn
subrtn	finmsg, parse, sparpt

include "cdefio.i"	# for lsfid
include "coutln.i"	# for fofid, tranuc
include "cerr.i"	# for errtot

    character infn(MAXPATH), outfn(MAXPATH), fullfn(MAXPATH), filsep
    integer junk, larg, outfid, srcfid
    logical dolist, dobanr

ifdef MSFORT4
    string inft SRCEXT, outft FOREXT, listft LSTEXT,
	   banner "Harrisoft Ratfor/77 4.44.5	 Dec 2017\n",
	   copyrt "Copyright (C) 1999-2017 Michael J. Harrison\n\n",
	   header "C...  Harrisoft Ratfor/4.44.5 dd-mmm-yy hh:mm:ss"
		  #000000000111111111122222222223333333333444444444
		  #123456789012345678901234567890123456789012345678
	   define RFDSTRT 31	#----------------^
endif
ifndef MSFORT4
    string inft SRCEXT, outft FOREXT, listft LSTEXT,
	   banner "Harrisoft Ratfor/77 4.44.5	 Dec 2017\n",
	   header "C...  Ratfor/4.44.5 dd-mmm-yy hh:mm:ss"
		  #000000000111111111122222222223333333333444444444
		  #123456789012345678901234567890123456789012345678
	   define RFDSTRT 21	#------^
endif

    # Initialize
    dummy = 0		# Reference dummy to prevent compilation warning
    errtot = 0
    call initgp

    # Get options, argument.  Open input, FORTRAN output and optionally list.
    # Ratfor input is read in TEXT mode, FORTRAN output is written
    # in BINARY mode (unless actually going to STDOUT).  Listing
    # file is written in TEXT mode (so LF char gets turned into CR-LF).
    # No argument implies use STDIN, STDOUT and no listing.
    #
    dolist = NO
    dobanr = NO
    outfn(1) = EOS
    lsfid = -1
    filsep = FILSEP			# Force character type (INTEGER*1)
    larg = getarg(infn, MAXPATH)
    while (larg ^= EOF & infn(1) == MINUS) {
	if (infn(2) == ICHAR('l'))
	    dolist = YES
	else if (infn(2) == ICHAR('v'))
	    dobanr = YES
	else if (infn(2) == ICHAR('o')) {
	    if (infn(3) != EOS)
		junk = strcpy(outfn, infn(3))
	    else
		larg = getarg(outfn, MAXPATH)
	    }
	larg = getarg(infn, MAXPATH)
	}

    # Banner and (DOS only) Copyright Notice
ifndef MSFORT4
    if (dobanr)
	call putlin(banner, STDERR)
endif
ifdef MSFORT4
    call putlin(banner, STDERR)
    call putlin(copyrt, STDERR)
endif

    if (larg == EOF) {
	srcfid = STDIN
	if (outfn(1) != EOS) {
	    junk = strcpy(fullfn, outfn)
	    outfid = open(fullfn, OUTPUT + BINMODE)
	    if (outfid < 0) {
		call cant(outfid, fullfn)
		errtot = 2
		}
	    }
	else {
	    outfid = STDOUT
	    }
	fullfn(1) = EOS
	}
    else {
ifdef MSFORT4
	call trn2uc(infn)
endif
	call rfmkfn(infn, inft, fullfn) # Make full file name (ensures FILSEP)
	srcfid = open(fullfn, INPUT)	# Open main input file
	if (srcfid < 0) {
	    call cant(srcfid, fullfn)
	    errtot = 1
	    }
	else {
	    if (outfn(1) != EOS)
		junk = strcpy(fullfn, outfn)
	    else {
		fullfn(strchr(fullfn, filsep)) = EOS # Remove .ext
		call rfmkfn(fullfn, outft, fullfn)
	    }
	    outfid = open(fullfn, OUTPUT + BINMODE) # Open FORTRAN output file
	    if (outfid < 0) {
		call cant(outfid, fullfn)
		call close(srcfid)		    # If error, close input
		errtot = 2
		}
	    else {
		if (dolist) {	    # If he wants listing, open file
		    fullfn(strchr(fullfn, filsep)) = EOS	# Isolate ...
		    call rfmkfn(fullfn, listft, fullfn) 	#  file prefix
		    lsfid = open(fullfn, OUTPUT)
		    if (lsfid < 0) {		    # If we fail,
			call cant(lsfid, fullfn)    # warn
			lsfid = -1		    # but carry on without
			}
		    }
		}
	    }
	}

    # If files opened, translate Ratfor program
    if (errtot == 0) {
	fofid = outfid		# File i/d for FORTRAN output
	call initio		# All files open, away we go ...
	call initkw
	call initlu
	call initfs
	junk = labgen(0)
	call rfdate(header(RFDSTRT))
	tranuc = NO		# Don't upper-case header line
	call outstr(header)
	call outdon
	tranuc = YES		#  but do the rest
	call opnget(srcfid)
	call parse(srcfid)
	if (lsfid > 0) {
	    call sparpt 	# Suppress this normally (7/18/80)
	    call close(lsfid)			#   and close
	    }
	if (outfid == STDOUT)
	    call putch(EOF, outfid)		# Flush STDOUT
	else {
	    call close(outfid)			#   and close named file
	    }
	junk = clnget(srcfid)
	call close(srcfid)
	if (errtot > 0)
	    call finmsg(fullfn, errtot)
	}

    ratfor = errtot	# Return value
    return

    end # of ratfor
