# strpk - routines for "string" statement savdat, strdat, string
#
# Copyright (c) 1988 by Michael J. Harrison
#
# Revised MJH, 24-May-86: strdat fixed to allow true CHARACTER variables
# Revised MJH, 24-May-86: Support real CHARACTER variables
# Revised MJH, 10-Jan-87: Switch to str... string functions
# Revised MJH, 22-Mar-87: Deal with MS FORTRAN non-support of null char const
# Revised MJH,  8-Sep-88: Output integers in DATA stmt for INTEGER*1 chars
# Revised MJH, 29-Dec-90: Adapt to changed interface for lookup
# M.J.Harrison 10-Apr-98: Linux

include dmach	# for MSFORT4
include dstd
include drmax
include drmeta

#----------------------------------------------------------------------
# cnvlen - compute actual length of string array (allowing for \...)
#----------------------------------------------------------------------

integer function cnvlen(str)
character str(ARB)

charfunc cnvesc

    integer i, len
    character cjunk

    len = 0
    for (i = 1; str(i) ^= EOS; i = i + 1) {
	len = len + 1
	if (str(i) == ESCFLAG)
	    cjunk = cnvesc(str, i)
	}

    cnvlen = len - 1		# allow for closing quote
    return
    end

#----------------------------------------------------------------------
# savdat - save "string" data for DATA statement(s)
#----------------------------------------------------------------------

subroutine savdat(varnm, varstr)
character varnm(MAXVNAME), varstr(ARB)

intfunc strcpy, strlen
include cstrda

    integer junk, lvarnm, lvarst

    lvarnm = strlen(varnm) + 1	# allow for EOS
    lvarst = strlen(varstr) - 1 # omit enclosing quotes
    if (sdfree + lvarnm + lvarst > MAXSTRS)
	call error("String data area full (savdat).")
    else {
	junk = strcpy(sddata(sdfree), varnm)	# save var name
	sdfree = sdfree + lvarnm
	varstr(lvarst + 1) = EOS		# clobber closing quote
	junk = strcpy(sddata(sdfree), varstr(2))# skip opening quote
	sdfree = sdfree + lvarst
	varstr(lvarst + 1) = varstr(1)		# put back closing quote
	}

    return
    end

#----------------------------------------------------------------------
# strdat - output DATA statement(s) for "string" statement(s)
#----------------------------------------------------------------------

subroutine strdat

charfunc cnvesc
intfunc strlen
include cstrda

    integer i, ic
    character c, sepch, slash
    string datach "DATA"

    slash = SLASH
    i = 1
    while (i < sdfree) {
	call outtab
	call outstr(datach)	    # "DATA"
	call outstr(sddata(i))	    # "varname"
	i = i + strlen(sddata(i))   # move to EOS
	sepch = SLASH

	repeat {
	    i = i + 1
	    c = sddata(i)
	    call outch(sepch)	    # "/" (first), "," (rest)
	    sepch = COMMA
	    if (c == ESCFLAG)	    # convert \... sequences
		ic = cnvesc(sddata, i)
	    else
		ic = c
	    call outnum(ic)
	    } until (c == EOS)

	call outch(slash)
	call outdon
	i = i + 1		    # move past EOS
	}
    sdfree = 1			    # clear data area

    return
    end

#----------------------------------------------------------------------
# string - process the "string" statement
#----------------------------------------------------------------------

subroutine string(tokstr, toklen)
integer toklen
character tokstr(toklen)

intfunc cnvlen, gettok, lookup, strcpy
logfunc rchknm

    character varnm(MAXVNAME), comma, lparen, rparen
    integer junk, t
    string cchar "character", xchar "INTEGERXX"

    comma = COMMA
    lparen = LPAREN
    rparen = RPAREN
    if (lookup(cchar, xchar, YES) == 0) # allow for define of "character"
	junk = strcpy(xchar, cchar)
    call outtab 		    # get to col 7
    call outstr(xchar)		    # "INTEGER" or "CHARACTER"
    t = gettok(tokstr, toklen)
    while (t ^= EOF & t ^= NEWLINE) {
	if (^rchknm(tokstr, varnm, MAXVNAME)) { 	    # if error,
	    repeat					    # flush to COMMA
		t = gettok(tokstr, toklen)		    # or NEWLINE
		until (t == EOF | t == COMMA | t == NEWLINE)
	    }
	else {
	    t = gettok(tokstr, toklen)
	    if (t ^= SQUOTE & t ^= DQUOTE)
		call synerr("Quoted string missing.")
	    else {
		call outstr(varnm)		# "name("
		call outch(lparen)
		call outnum(cnvlen(tokstr))	# "len"
		call outch(rparen)		# ")"
		call savdat(varnm, tokstr)	# save for DATA stmt
		t = gettok(tokstr, toklen)
		}
	    }
	if (t == COMMA) {
	    call outch(comma)
	    t = gettok(tokstr, toklen)
	    if (t == NEWLINE)			# ",NL" signals continue
		t = gettok(tokstr, toklen)
	    }
	else if (t ^= NEWLINE) {
	    call synerr("Bad token after string.")
	    repeat				# flush stmt
		t = gettok(tokstr, toklen)
		until (t == NEWLINE | t == EOF)
	    }
	}
    call outdon     # finish off type stmts line
    call strdat     # output corresponding DATA statements

    return
    end
