#-----------------------------------------------------------------------
#
#  tokpk - Routines for reading tokens: gettok, rdtok
#	   Also, Block Data subprogram ctypbd to initialize type array
#
#	   Local routines: tokrep, twocht
#
#  Revised MJH, 26-May-86: Support real CHARACTER variables
#  Revised MJH, 31-May-86: Use common type array
#  Revised MJH,  9-Sep-88: For INTEGER*1 chars
#  Revised MJH, 30-Dec-90: New macro replacement algorithm
#  M.J.Harrison 10-Apr-98: Linux
#
#-------------- Copyright (C) 1986 by Michael J. Harrison --------------

include dstd
include ddefio
include drmax
include drmeta


#----------------------------------------------------------------------
# ctypbd - Block Data to initialize type array
#----------------------------------------------------------------------

block data ctypbd

include ctype

# DIGIT is 0 through 9 (decimal 48 through 57)
# ALPHA is A through Z (65 through 90) and a through z (97 through 122)
# NOTE: Underscore (decimal 95) counts as ALPHA.

    data type /  0,  1,  2,  3,  4,  5,  6,  7,
		 8,  9, 10, 11, 12, 13, 14, 15,
		16, 17, 18, 19, 20, 21, 22, 23,
		24, 25, 26, 27, 28, 29, 30, 31,
		32, 33, 34, 35, 36, 37, 38, 39,
		40, 41, 42, 43, 44, 45, 46, 47,
		DIGIT, DIGIT, DIGIT, DIGIT, DIGIT, DIGIT, DIGIT, DIGIT,
		DIGIT, DIGIT, 58,    59,    60,    61,	  63,	 63,
		64,    ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
		ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
		ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
		ALPHA, ALPHA, ALPHA, 91,    92,    93,	  94,	 ALPHA,
		96,    ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
		ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
		ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
		ALPHA, ALPHA, ALPHA, 123,   124,   125,   126,	 127 /

    end


#----------------------------------------------------------------------
# gettok - get token for Ratfor, doing macro replacement
#----------------------------------------------------------------------
# Revised MJH, 12-Jun-81: no longer translate any token to upper case
# Revised MJH,	8-Apr-88: Change macro replacement algorithm

integer function gettok(lexstr, toksiz)
character lexstr(ARB)
integer toksiz

include cline
intfunc lookup, rdtok

    integer repidx
    character repstr(MAXDEF)

    repeat {		# Keep going until we get a non-null token
	gettok = rdtok(lexstr, toksiz)	    # get token
	if (^exptok) {
	    if (gettok == EOPUTBK) {	    # end of expanded macro
		exptok = YES
		gettok = rdtok(lexstr, toksiz)
		}
	    }
	if (exptok & gettok == ALPHA) {
	    repidx = lookup(lexstr, repstr, YES)
	    if (repidx != 0) {
		if (repstr(1) ^= EOS) {
		    call deftog(repidx)     # mark symbol inactive
		    call rescan(lexstr, toksiz, repstr)
		    call deftog(repidx)     # mark symbol active again
		    exptok = NO 	    # don't expand again
		    }
		lexstr(1) = EOS		# force reread
		}
	    }
	} until (lexstr(1) ^= EOS | gettok == EOF)

    return
    end


#----------------------------------------------------------------------
# rdtok - read token for Ratfor pre-processor
#----------------------------------------------------------------------
# Revised MJH, 23-Dec-85: Remove line counting (to ngetc/putbak)

integer function rdtok(lexstr, toksiz)
integer toksiz
character lexstr(toksiz)

include cline
include ctype
intfunc ngetc

    character c, newlin
    integer delim1, delim2, i, ic, icq, t

    newlin = NEWLINE

    # Position at next non-white-space char
    ic = ngetc(c)
    while (ic == SPACE | ic == TAB | ic == FORMFEED)
	ic = ngetc(c)

    # Scan the next token: a token is a name, a number, a quoted string,
    # or maybe a two-char combo (like ^=).  Also, skip comment to NEWLINE.
    i = 1			# Next free char in lexstr
    if (ic == EOF)
	rdtok = EOF
    else if (ic == EOPUTBK)
	rdtok = EOPUTBK
    else {			# Switch on type
	t = c
	t = type(1 + IAND(t, CHARMASK))
	if (t == ALPHA | t == DIGIT) {	# Name or number, set delimiter
	    rdtok = t
	    delim1 = DIGIT		# digit never delimiter
	    if (t == ALPHA)		# if 1st char alpha
		delim2 = ALPHA		# neither ALPHA nor digit delimits
	    else			# otherwise, ALPHA is delimiter
		delim2 = DIGIT
	    repeat {			# Scan to non-alphanumeric, or too big
		lexstr(i) = c
		ic = ngetc(c)
		t = type(1 + ic)
		i = i + 1
		} until ((t ^= delim1 & t ^= delim2) |
			 ic == EOF | ic == EOPUTBK | i == toksiz)
	    if (i == toksiz)
		call synerr("Token too long.")
	    if (ic ^= EOF) {
		if (ic == EOPUTBK)
		    call pbmark
		else
		    call putbak(c)	# We overshot by 1
		}
	    }
	else if (ic == SQUOTE | ic == DQUOTE) {
	    rdtok = t
	    icq = ic			# For matching closing quote
	    lexstr(i) = c		# i.e. lexstr(1)
	    i = i + 1
	    ic = ngetc(lexstr(i))
	    i = i + 1
	    while (ic ^= icq & ic ^= NEWLINE &
		   ic ^= EOF & ic ^= EOPUTBK & i < toksiz) {
		if (ic == BACKSL) {	# \ escapes either quote and itself
		    ic = ngetc(c)	# Look ahead
		    if (ic == EOF)
			break
		    if (ic == EOPUTBK)	# If mark flushed, replace it
			call pbmark
		    else if (ic ^= SQUOTE & ic ^= DQUOTE & ic ^= BACKSL)
			call putbak(c)
		    else {
			lexstr(i) = c
			i = i + 1
			}
		    }
		ic = ngetc(lexstr(i))
		i = i + 1
		}
	    if (i == toksiz)		    # Too long, flag
		call synerr("Token too long.")
	    else if (ic == NEWLINE | ic == EOF | ic == EOPUTBK) {
		call synerr("Missing quote.")   # String may not span lines
		lexstr(i - 1) = lexstr(1)	#  so flag and recover
		if (ic == EOPUTBK)
		    call pbmark
		call putbak(newlin)
		}
	    }
	else if (ic == SHARP) { 	# strip comments
	    repeat {
		ic = ngetc(c)
		} until (ic == NEWLINE | ic == EOF | ic == EOPUTBK)
	    if (ic == NEWLINE) {
		lexstr(i) = c
		i = i + 1
		}
	    rdtok = ic
	    }
	else {
	    rdtok = ic
	    lexstr(i) = c
	    call twocht(i, lexstr)	# look for 2-char token
	    i = i + 1
	    }
	# end switch
	}
    lexstr(i) = EOS

    return
    end


#----------------------------------------------------------------------
# rescan - rescan replacement string once, recursing if necessary
#----------------------------------------------------------------------

subroutine rescan(lexstr, toksiz, repstr)
character lexstr(ARB)
integer toksiz
character repstr(MAXDEF)
#
# Returns: Evaluated replacement string pushed back onto input.

include cline
intfunc*4 evali
intfunc clnget, itoc, lookup, ngetc, rdtok, strcpy, strlen
intfunc rpwrit

    integer*4 tokval
    integer errcod
    integer junk, lookix, repsp, repidx(MAXMAC)
    integer repbp, tok
    character c, newlin, repbuf(BUFSIZE)
    string nullms "null string replacement"

    newlin = NEWLINE
    call pbmark			# mark end of replacement string
    call pbstr(repstr)		# put it back for rescan
    repbp = 1			# buffer pointer
    repbuf(repbp) = EOS		# init buffer empty
    repsp = 0			# macro stack pointer
    while (repsp >= 0) {
	tok = rdtok(lexstr, toksiz)
	if (tok == EOPUTBK) {		# end of replacement string
	    if (repsp > 0) {
		junk = clnget(PSEUDOFID)
		call deftog(repidx(repsp))
		}
 	    repsp = repsp - 1
	    }
	else if (tok != ALPHA) {	# can't match, copy to buffer
	    repbuf(repbp) = SPACE	# token separator
	    repbp = repbp + 1
	    repbp = repbp + strcpy(repbuf(repbp), lexstr)
	    }
	else {				# might match
	    lookix = lookup(lexstr, repstr, YES)
	    if (lookix != 0) {		# if macro replacement
		if (repstr(1) == EOS) {	# null replacement
		    junk = rpwrit(1, nullms, strlen(nullms))
		    }
		else {
		    repsp = repsp + 1	# recurse
		    if (repsp > MAXMAC)
			call error("Max macro nesting depth exceeded.")
		    repidx(repsp) = lookix
		    call deftog(lookix)	# rescan
		    call opnget(PSEUDOFID)
		    call pbmark
		    call pbstr(repstr)
		    }
		}
	    else {			# no match, copy to buffer
		repbuf(repbp) = SPACE
		repbp = repbp + 1
		repbp = repbp + strcpy(repbuf(repbp), lexstr)
		}
	    }
	}
    call pbmark 		# mark end of eval string
    call putbak(newlin)		# to terminate evali's scan
    call pbstr(repbuf)		# set up to evaluate
    tokval = evali(lexstr, toksiz, errcod)
    if (errcod == 0)		# valid expression, use value
	errcod = itoc(tokval, repbuf, MAXCHARS)
    while (ngetc(c) != EOPUTBK)	# flush to mark
	;
    call pbmark			# mark end of expanded macro
    call pbstr(repbuf)

    return
    end


#----------------------------------------------------------------------
# twocht - deal with 2-char tokens for Ratfor, translate to std. char set
#----------------------------------------------------------------------
# Revised MJH,	2-Feb-86: Allow ! for ^, != for ^=, retire ! for |
# Revised MJH, 31-May-86: Retire << and >> braces
# Revised MJH, 10-Jan-87: Switch to str... string functions

subroutine twocht(i, lexstr)
integer i
character lexstr(ARB)

intfunc ngetc, strchr

    integer ic, j
    character c
    string char1 "><=^!",  stdchs "><=^^"

    j = strchr(char1, lexstr(i))
    if (j > 0) {			# might be 2-char token
	lexstr(i) = stdchs(j)		# translate to std set
	ic = ngetc(c)
	if (ic ^= EOF) {		# look ahead
	    if (ic == EOPUTBK)		# if we hit mark
		call pbmark		# replace it
	    else if (c == EQUALS) {	# composite operator, so
		i = i + 1		# pick up both chars
		lexstr(i) = c
		}
	    else			# not composite operator, so
		call putbak(c)		# put back second char
	    }
	}

    return
    end
