#-----------------------------------------------------------------------
#
#  getch      - Get next character from file
#
#  Synopsis:	int getch(c, handle)
#		char	c	also returned in arg
#		int	handle	file handle (was returned by open)
#		Returns: next character (as int)
#
#  Notes:	Read code repeated in CR-LF lookahead for speed (no S/R call)
#
#  Written MJH, 27-Nov-85:  Version with UNIX-size buffers
#  Revised MJH, 29-Nov-85:  For TEXTMODE files, return EOF for CTRL-Z
#  Revised MJH,  4-Dec-85:  Do write of dirty buffer before reading into it
#  Revised MJH, 25-Jan-86:  Change handling of dospos (now pos of buffer(1))
#  Revised MJH, 23-May-86:  Adapt for true CHARACTER variables
#  Revised MJH, 16-Feb-87:  Use rp.... instead of pf....
#  Revised MJH, 22-Jul-87:  Replace CR, LF with metacharacter equivalents
#  Revised MJH, 21-Dec-87:  Implement uniform file system
#  Revised MJH,  9-Sep-88:  For INTEGER*1 chars, ensure returned value
#			     is masked to 8 bits (in low order byte)
#  M.J.Harrison 10-Apr-98:  Linux
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

include dmach
include dstd
include dmetac	    # for CR, LF
include dstdio	    # for BINMODE
include derrno

integer function getch(c, handle)
character c
integer   handle

intfunc rpread, rpwrit
charfunc int2ch
subrtn	error

include cpfile
include cerrno

    integer bufidx	# index to buffer in pfilb
    integer idx 	# index to file control block
    integer lread	# length actually read (<= BUFSIZE)
    integer lwrit	# length written by rpwrit
    integer ic		# char as integer

    # Set index (handle starts at 0, index must start at 1).
    # If buffer empty, read next buffer load.

    idx = handle + 1
    bufidx = bufadr(idx)
    if (nextc(idx) < curcnt(idx))
	lread = 1			# Positive value to pass test below
    else {

	# Write out dirty buffer before clobbering buffer area.

	if (wflag(idx)) {		# curcnt must be > 0
	    lwrit = rpwrit(dosfid(idx), pfilb(bufidx), curcnt)
	    if (lwrit <= 0) {
		if (lwrit == -REDSKFULL)
		    call error("Disk full.")
		else
		    call error("File write error (getch).")
	    }
	}

	lread = rpread(dosfid(idx), pfilb(bufidx), bufsiz(idx))
	if (lread <= 0) {		# EOF or read error
	    ic = EOF
	    if (lread < 0)		# Error, set errno
		errno = -lread
	} else {			# Read successful, reset buffer
	    nextc(idx) = 0
	    curcnt(idx) = lread
	    wflag(idx) = NO		# Virgin buffer
	    dospos(idx) = dospos(idx) + 1
	}
    }

    if (lread > 0) {			# No EOF or error
	ic = pfilb(bufidx + nextc(idx))
	ic = IAND(ic, BYTEMASK)
	nextc(idx) = nextc(idx) + 1

	# Special processing for TEXTMODE.

	if (omode(idx) < BINMODE) {
	    if (ic == CR) {		# Look for CR-LF
		if (nextc(idx) == curcnt(idx)) {# CR was last char in buffer
		    lread = rpread(dosfid(idx), pfilb(bufidx), bufsiz(idx))
		    if (lread <= 0) {		# EOF or read error
			ic = EOF
			if (lread < 0)		# Error, set errno
			    errno = -lread
		    } else {			# Read successful, reset buffer
			nextc(idx) = 0
			curcnt(idx) = lread
			wflag(idx) = NO 	# Virgin buffer
			dospos(idx) = dospos(idx) + 1
		    }
		}
		if (ic == EOF)
		    ic = CR
		else {
		    ic = pfilb(bufidx + nextc(idx))
		    ic = IAND(ic, BYTEMASK)
		    nextc(idx) = nextc(idx) + 1
		    if (ic ^= LF) {
			ic = CR 	    # CR not followed by LF: back up
			nextc(idx) = nextc(idx) - 1
		    }
		}
	    } # end if (ic == CR)
	} # end special processing for TEXTMODE

    } # end if (lread > 0)

    c = int2ch(ic)		# Return char in argument
    getch = ic			# and metachar as function value
    return

    end 	# of getch
