#-----------------------------------------------------------------------
#
#  open - Open a named file and return its handle
#
#  Synopsis:	int open(fname, fmode)
#		char(*) fname	filename (full pathname or less)
#		int	fmode	O_RDONLY 0x0000  (values can be OR'ed or added)
#				O_WRONLY 0x0001
#				O_RDWR	 0x0002
#				O_TEXT	 0x0000  text (CR/LF <--> NEWLINE) dflt
#				O_BINARY 0x0004  binary (CR/LF untranslated)
#				O_APPEND 0x0008  Ignored (use seek to end)
#				O_CREAT  0x0100  Ignored (implied by r/w)
#				O_TRUNC  0x0200  Ignored (implied by r/w)
#				O_EXCL	 0x0400  Ignored
#		Returns: >= 0, handle; -1, error (code in errno)
#
#  Notes:	For read-only, file must exist.  For write-only, file is
#		truncated (if it exists) or created (if it does not).  For
#		read/write, file need not exist; if it does it is opened
#		and the file pointer positioned at the first byte, if it
#		does not it is created.
#
#  Written MJH, 27-Nov-85:  Version with UNIX-size buffers
#  Revised MJH, 25-Jan-86:  Use errno; also, new treatment of dospos
#  Revised MJH, 16-Feb-87:  Use rpopen instead of pfopen
#  Revised MJH, 21-Dec-87:  Implement uniform file system
#  M.J.Harrison 04/16/1998: Linux: create after any error code (<0)
#  Mike Harrison 04/19/2014: Bring back BINMODE for MinGW
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

include dstd
include dstdio
include dmach	    # for file/path lengths
include derrno


integer function open(fname, fmode)
character fname(ARB)
integer fmode

intfunc isatty, rpopen

include cpfile
include cerrno

    integer handle,	# our file handle
	    idx,	# index to file control block
	    mode,	# open mode, including BIN/TEXT flag
	    retval,	# our return value
	    rwmode	# open mode, r/w/rw part only


    # Find first free handle

    for (idx = 1; idx <= _NFILE & dosfid(idx) ^= FREESLOT; idx = idx + 1)
	;

    # If none available, return appropriate error code, else continue

    if (idx > _NFILE) {
	errno = RENOHNDLS
	retval = -1
    } else {

	# Adjust our handle from FORTRAN array addressing
	# Get fmode r/w/rw and translate bits

	handle = idx - 1

	mode = fmode - 8 * (fmode / 8)
	rwmode = mode
	if (mode >= BINMODE)
	    rwmode = mode - BINMODE

	# Attempt to open file

	if (rwmode == OUTPUT)
	    retval = rpopen(fname, CRFLAG + mode)	# i.e. create
	else {
	    retval = rpopen(fname, mode)
	    if (retval < 0 & rwmode == INOUT)
		retval = rpopen(fname, CRFLAG + mode)
	}

	# If failure, set errno and return -1
	# If successful, set up corresponding file control block
	# and return our handle

	if (retval < 0) {
	    errno = -retval
	    retval = -1
	} else {
	    nextc(idx) = 0
	    curcnt(idx) = 0	    # Forces actual read, if reading
	    omode(idx) = mode
	    devflg(idx) = isatty(retval)
	    dosfid(idx) = retval
	    wflag(idx) = NO
	    dospos(idx) = -1	    # So first read sets to 0

	    retval = handle
	}
    }

    open = retval
    return

    end 	# of open
