#-----------------------------------------------------------------------
#
#  putch - Put next character into file
#
#  Synopsis:	call putch(ic, handle)
#		int	ic	character to put, as integer
#		int	handle	file handle (was returned by open)
#
#  Written MJH, 27-Nov-85:  Version with UNIX-size buffers
#  Revised MJH, 30-Nov-85:  Keep up buffer high water mark for seek
#  Revised MJH,  4-Dec-85:  Make random access read/write work
#  Revised MJH, 25-Jan-86:  New treatment of dospos
#  Revised MJH, 23-May-86:  Adapt for true CHARACTER variables
#  Revised MJH,  6-Jun-86:  Output destination changed from STDERR
#  Revised MJH, 16-Feb-87:  Use rp.... instead of pf....
#  Revised MJH, 22-Jul-87:  Replace CR, LF with metacharacter equivalents
#  Revised MJH, 21-Dec-87:  New buffer sizes, unified file system
#  M.J.Harrison 10-Apr-98:  Remove INT4 for Linux
#  M.J.Harrison 30-Dec-16:  Use INTEGER*8 for seek offset for 64-bit addressing
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

include dstd
include dstdio
include dmetac	    # for CR, LF
include dmach	    # For STDOUT
include derrno

subroutine putch(ic, handle)
integer ic
integer handle

intfunc*4 rpseek
intfunc putcwr
charfunc int2ch
subrtn error

include cpfile

    integer*8 i8
    integer buflen	# Length of buffer(idx)
    integer idx 	# Index to file control block
    integer lwrit	# Length written
    character c 	# Real char for storing into buffer

    # Set index (handle starts at 0, index must start at 1).
    # If buffer dirty and it's full or we need to flush, write it.
    # By happy coincidence, curcnt is number of bytes to write.

    idx = handle + 1
    buflen = bufsiz(idx)
    lwrit = 1		# make sure it has some positive value

    if (wflag(idx) &
	(nextc(idx) == buflen | (ic == EOF & curcnt(idx) > 0)))
	lwrit = putcwr(idx, curcnt(idx))

    # Insert char into buffer, dealing with NEWLINE => CR, LF and buffering.

    if (lwrit > 0 & ic ^= EOF) {

	# If this is very first byte written to buffer, update dospos;
	# But, if buffer had been filled by a read (so this is first write
	# after that), we must seek DOS to start of buffer (read left file
	# pointer one past end of buffer but set dospos correctly).

	if (^wflag(idx)) {
	    if (curcnt(idx) == 0)
		dospos(idx) = dospos(idx) + 1
	    else {
		i8 = dospos(idx)	# Force integer*4
		i8 = i8 * buflen
		if (rpseek(dosfid(idx), i8, 0) < 0)
		    call error("Seek error (putch).")
	    }
	}

	c = int2ch(ic)
	if (lwrit > 0 & omode(idx) < BINMODE) {	# TEXTMODE file
	    if (ic == NEWLINE) {	# NEWLINE => CR, LF
		pfilb(bufadr(idx) + nextc(idx)) = CR
		nextc(idx) = nextc(idx) + 1
		if (nextc(idx) > curcnt(idx))	# Maintain high water mark
		    curcnt(idx) = nextc(idx)
		if (nextc(idx) == buflen) {	# CR might have filled buffer
		    lwrit = putcwr(idx, buflen)
		    dospos(idx) = dospos(idx) + 1   # c will be first in buf
		}
		c = LF
	    }
	} # end if TEXTMODE

	if (lwrit > 0) {
	    pfilb(bufadr(idx) + nextc(idx)) = c
	    nextc(idx) = nextc(idx) + 1
	    if (nextc(idx) > curcnt(idx))   # Maintain high water mark
		curcnt(idx) = nextc(idx)
	    wflag(idx) = YES

	    # If writing to STDxxx, NEWLINE flushes buffer,
	    # if we are writing to a device.
	    # Here nextc(idx) is guaranteed > 0

	    if (ic == NEWLINE & dosfid(idx) <= STDMAX & devflg(idx) == 1)
		lwrit = putcwr(idx, nextc(idx))
	}
    } # not EOF

    return

    end 	# of putch

#-----------------------------------------------------------------------
#  putcwr - write a buffer for putch (internal function)
#-----------------------------------------------------------------------

integer function putcwr(idx, lw)	# returns rpwrit error code
integer idx				# index to file control block
integer lw				# length to write

intfunc rpwrit
subrtn error

include cpfile
include cerrno

    integer lwrit

    lwrit = rpwrit(dosfid(idx), pfilb(bufadr(idx)), lw)
    if (lwrit < 0) {
	errno = -lwrit
	if (lwrit == -REDSKFULL)
	    call error("Disk full.")
	else
	    call error("File write error (putch).")
    }
    nextc(idx) = 0
    curcnt(idx) = 0
    wflag(idx) = NO

    putcwr = lwrit
    return

    end 	# of putcwr
