# SCNINT      - Determine if string is integer,return value if so (RATUTI)
#
# Usage:	isint(YES/NO) = scnint(string, pos, nxtpos, intval)
#
# Purpose:	Used to recognize string of digits terminated by non-digit
#
# Input:	string	- char string containing string in question
#		pos	- starting position in char string
# Output:	Value	- YES if string(pos)... is integer, else NO
#		nxtpos	- position immediately following integer
#		intval	- value of integer, or 0 if not
#
# Method:	Scan until non-digit found; accumulate value.
#		Test for digit depends on chars for 0 through 9 being
#		consecutive with no gaps (true for ASCII and EBCDIC).
#		Computation of total depends on doing arithmetic on
#		characters.  Will recognize integer if string(pos) is digit,
#		whatever follows.  Relies on FORTRAN to handle overflow.

# Copyright (C) 1983 by Michael J. Harrison
# Written MJH,	17-Feb-83
# Revised MJH,	15-Mar-83: name changed from PEISI to SCNINT
# Revised MJH,   8-Sep-88: Accommodate INTEGER*1 characters with ichar()
# M.J.Harrison   3-Apr-98: Linux: character comparisons

include dstd

logical function scnint(string, pos, nxtpos, intval)
character string(ARB)
integer nxtpos, pos
integer*4 intval

	scnint = NO		# assume not integer
	intval = 0		# for loop test works even if
	for (nxtpos = pos;	#  sign extension
	     string(nxtpos) >= ICHAR('0') & string(nxtpos) <= ICHAR('9');
	     nxtpos = nxtpos + 1) {	# scan until non-digit
		scnint = YES		# we hit a digit
		intval = 10 * intval + string(nxtpos) - ICHAR('0')
	}

	return
	end
