#-----------------------------------------------------------------------
#
#  seek       - Reposition file pointer and return new value
#  Synopsis:	pos = seek(handle, offset, origin)
#		int*4	pos	set to new file position (offset from start)
#		int	handle	file handle (was returned by open)
#		int*4	offset	byte offset from origin
#		int*2	origin	0: start; 1: current; 2: end
#
#  Notes:	Note that return value and offset are 32-bit integers
#
#  Written MJH, 30-Nov-85: Version with UNIX-size buffers
#  Revised MJH, 25-Jan-86: New treatment of dospos
#			   Report disk full error from write
#  Revised MJH, 23-May-86: Adapt for true CHARACTER variables
#  Revised MJH,  5-Apr-87: pf... --> rp...
#  Revised MJH, 21-Dec-87: New buffer sizes, unified file system
#  M.J.Harrison 10-Apr-98: Linux
#  M.J.Harrison 30-Dec-16: INTEGER*8 offset for seek with 64-bit addressing 
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

include dstd
include dmach
include derrno

integer*4 function seek(handle, offset, origin)
integer   handle
integer*4 offset
integer   origin

intfunc*4 rpseek
intfunc   rpread, rpwrit
subrtn	  error

include cpfile
include cerrno

    integer*4 i4	# temp long int
    integer*4 lint	# long integer for argument lists
    integer*8 offs64	# 64-bit integer for fseek offset
    integer*4 pos	# DOS file position
    integer block	# our block (BLKSIZ long)
    integer buflen	# length of buffer(idx)
    integer displ	# displacement within block
    integer idx 	# index to file control block
    integer rpcod	# return code from rpread/rpwrit
    logical mustw	# YES iff must write out buffer

    # Allow for handle offset from open.
    # Convert dospos (buffer blocks) to file pos (bytes).
    # Negative dospos denotes freshly opened file with no I/O yet.

    idx = handle + 1
    pos = dospos(idx)
    buflen = bufsiz(idx)
    if (pos < 0)
	pos = 0
    else
	pos = pos * buflen

    # If file not open, return error
    # Else if ftell call, make special case (for efficiency)
    # Else do all the work.

    rpcod = 0
    if (dosfid(idx) == FREESLOT)
	pos = -1
    else if (offset == 0 & origin == 1)
	pos = pos + nextc(idx)
    else {

	# If origin = 0 or 1, see if we are staying within current buffer
	# (if so, need not write it out before seeking).

	mustw = YES			# Pessimist!
	if (origin == 0 | origin == 1) {
	    i4 = pos			# Save for comparison
	    if (origin == 0)
		pos = offset
	    else if (origin == 1)
		pos = pos + nextc(idx) + offset
	    if (pos >= i4 & pos < (i4 + buflen))
		mustw = NO
	} else if (origin ^= 2)
	    call error("Seek origin not 0, 1, or 2.")

	# If writing and dirty buffer, must write as much as we have.
	# curcnt holds high water mark of real bytes in block.

	if (mustw & wflag(idx)) {
	    rpcod = rpwrit(dosfid(idx), pfilb(bufadr(idx)), curcnt(idx))
	    if (rpcod == -REDSKFULL)
		call error("Disk full.")
	}

	if (rpcod < 0) {		# Other write errors
	    errno = -rpcod
	    pos = -1
	} else {

	    # Compute block and displacement within it.

	    if (origin == 2) {		# Have to find EOF pos first
		lint = 0
		offs64 = lint
		pos = rpseek(dosfid(idx), offs64, 2)
		if (pos >= 0)
		    pos = pos + lint
		else {
		    rpcod = pos
		    errno = -rpcod
		    pos = -1
		}
	    }
	    if (rpcod >= 0) {
		block = pos / buflen	# pos and block both start at zero
		displ = pos - buflen * block

		# If staying within current buffer, just reset nextc.
		# Otherwise, seek to required pos for block, read it in.

		if (^mustw)
		    nextc(idx) = displ
		else {
		    offs64 = block	# Force integer*8
		    offs64 = offs64 * buflen
		    lint = rpseek(dosfid(idx), offs64, 0)
		    if (lint < 0) {
			errno = -lint
			pos = -1
		    } else {
			rpcod = rpread(dosfid(idx),
				       pfilb(bufadr(idx)), buflen)
			if (rpcod < 0) {
			    errno = -rpcod
			    pos = -1
			} else {
			    curcnt(idx) = rpcod
			    nextc(idx) = displ
			    wflag(idx) = NO
			    dospos(idx) = block
			}
		    }
		} # seek and read
	    } # rpcod >= 0
	} # no write errors
    } # not ftell

    seek = pos
    return

    end 	# of seek
